from django.shortcuts import render
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from account.models import Account
from androidapi.serializers.account_serializer import LoginSerializer
from colors.models import Color
from products.models import Product

class LoginAPIView(APIView):
    def post(self, request, *args, **kwargs):
        serializer = LoginSerializer(data=request.data)
        if serializer.is_valid():
            user = serializer.validated_data['user']

            # Prepare roles from user instance
            roles = user.get_roles()

            # Return user details with success = True
            return Response({
                "success": True,
                "status": "success",
                "message": "Login successful.",
                "data": {
                    "user_id": user.id,
                    "email": user.email,
                    "first_name": user.first_name,
                    "second_name": user.second_name,
                    "phone_number": user.phone_number,
                    "roles": roles,
                    "gender": user.gender,
                    "marital_status": user.marital_status,
                    "profile_image_url": user.profile_image.url if user.profile_image else None,
                    "created_at": user.created_at,
                }
            }, status=status.HTTP_200_OK)

        # Handle errors explicitly
        email = request.data.get('email')
        password = request.data.get('password')

        if not email or not password:
            return Response({
                "success": False,
                "status": "error",
                "message": "Email and password are required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            user = Account.objects.get(email=email)
        except Account.DoesNotExist:
            return Response({
                "success": False,
                "status": "error",
                "message": "User not found. Please check the email address."
            }, status=status.HTTP_404_NOT_FOUND)

        # Check for invalid password
        if not user.check_password(password):
            return Response({
                "success": False,
                "status": "error",
                "message": "Invalid password. Please try again."
            }, status=status.HTTP_401_UNAUTHORIZED)

        # Default error response
        return Response({
            "success": False,
            "status": "error",
            "message": "Login failed. Please try again.",
        }, status=status.HTTP_400_BAD_REQUEST)



from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status 

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status 
class GetUserInfoAPIView(APIView):
    def get(self, request, *args, **kwargs):
        user_id = request.query_params.get('user_id')

        if not user_id:
            return Response({
                "success": False,
                "status": "error",
                "message": "User ID is required."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            user = Account.objects.get(id=user_id)
        except Account.DoesNotExist:
            return Response({
                "success": False,
                "status": "error",
                "message": "User not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Prepare roles from user instance
        roles = user.get_roles()  # Assuming get_roles is a method in the Account model

        # Return all user details including official and salary info with empty values if not available
        return Response({
            "success": True,
            "status": "success",
            "message": "User found.",
            "data": {
                "user_id": user.id,
                "email": user.email or "",
                "first_name": user.first_name or "",
                "second_name": user.second_name or "",
                "phone_number": user.phone_number or "",
                "roles": roles or "",
                "gender": user.gender or "",
                "dob": user.dob or "",
                "age": user.age or "",
                "marital_status": user.marital_status or "",
                "location": user.location or "",
                "country": user.country or "",
                "branch": user.branch.branch_name if user.branch else "",
                "employment_type": user.employment_type or "",
                "kra_pin": user.kra_pin or "",
                "status": user.status or "",
                "education": user.education or "",
                "nationality": user.nationality or "",
                "next_of_kin_name": user.next_of_kin_name or "",
                "next_of_kin_phone": user.next_of_kin_phone or "",
                "emergency_contact": user.emergency_contact or "",
                "emergency_contact_name": user.emergency_contact_name or "",
                "emergency_contact_phone": user.emergency_contact_phone or "",
                "emergency_contact_relationship": user.emergency_contact_relationship or "",
                "secondary_emergency_contact_name": user.secondary_emergency_contact_name or "",
                "secondary_emergency_contact_phone": user.secondary_emergency_contact_phone or "",
                "secondary_emergency_contact_relationship": user.secondary_emergency_contact_relationship or "",
                "average_rating": user.average_rating or "",
                "profile_image_url": user.profile_image.url if user.profile_image else "",
                "documents": user.documents.url if user.documents else "",
                "created_at": user.created_at or "",
                "updated_at": user.updated_at or "",
                "is_staff": user.is_staff,
                "is_admin_user": user.is_admin_user,
                "is_super_user": user.is_super_user,
                "is_sales": user.is_sales,
                "is_purchases": user.is_purchases,
                "is_profiling": user.is_profiling,
                "is_production": user.is_production,
                "is_inventory": user.is_inventory,
                "is_pos": user.is_pos,
                "is_accounting": user.is_accounting,
                "key": user.key or "",
                "groups": [group.name for group in user.groups.all()],
                
                # Official Information (always return empty string if not found)
                "position": "",  # Position is not in the model, return ""
                "department": "",  # Department is not in the model, return ""
                "company": "",  # Company is not in the model, return ""
                "manager": "",  # Manager is not in the model, return ""
                "branch": "",  # Branch is not in the model, return ""

                # Salary Information (always return empty string if not found)
                "basic_salary": "",  # Basic Salary is not in the model, return ""
                "bonus": "",  # Bonus is not in the model, return ""
                "deductions": "",  # Deductions is not in the model, return ""
                "net_salary": "",  # Net Salary is not in the model, return ""
            }
        }, status=status.HTTP_200_OK)


from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from .models import Attendance
from datetime import datetime, timedelta
from django.utils.timezone import now

class RecordAttendanceAPIView(APIView):
    def post(self, request, *args, **kwargs):
        # Extract user, action, and location from request data
        user_id = request.data.get('user')
        action = request.data.get('action')  # Either 'check_in' or 'check_out'
        location = request.data.get('location')  # Location during check-in or check-out
        datetime_str = request.data.get('datetime')  # Expecting 'dd/mm/yyyy HH:MM'

        if not user_id or not action or not datetime_str:
            return Response({"success": False, "message": "User ID, action, and datetime are required."}, 
                            status=status.HTTP_400_BAD_REQUEST)

        # Ensure the datetime is in the correct format
        try:
            datetime_obj = datetime.strptime(datetime_str, '%d/%m/%Y %H:%M')  # Parse the input format
            adjusted_datetime = datetime_obj - timedelta(hours=3)  # Adjust for timezone difference
        except ValueError:
            return Response({"success": False, "message": "Invalid datetime format. Use dd/mm/yyyy HH:MM."}, 
                            status=status.HTTP_400_BAD_REQUEST)

        date = adjusted_datetime.date()

        if action == 'check_in':
            # Find or create attendance record for check-in
            attendance, created = Attendance.objects.get_or_create(user_id=user_id, date=date)

            # If it's a new check-in, update the check-in time and location
            attendance.check_in_time = adjusted_datetime
            attendance.check_in_location = location
            attendance.save()

            message = "Attendance checked in successfully."
        elif action == 'check_out':
            # Find existing attendance record for check-out
            try:
                attendance = Attendance.objects.get(user_id=user_id, date=date)

                # If check-in exists, update check-out time and location
                if attendance.check_in_time and not attendance.check_out_time:
                    attendance.check_out_time = adjusted_datetime
                    attendance.check_out_location = location
                    attendance.save()

                    message = "Attendance checked out successfully."
                else:
                    message = "Cannot check out without checking in first."
                    return Response({"success": False, "message": message}, status=status.HTTP_400_BAD_REQUEST)
            except Attendance.DoesNotExist:
                message = "No check-in record found for this user on the specified date."
                return Response({"success": False, "message": message}, status=status.HTTP_404_NOT_FOUND)
        else:
            return Response({"success": False, "message": "Invalid action. Must be 'check_in' or 'check_out'."}, 
                            status=status.HTTP_400_BAD_REQUEST)

        # Return updated attendance information
        return Response({
            "success": True,
            "message": message,
        }, status=status.HTTP_200_OK)



from django.http import JsonResponse 

def get_colors_and_products(request):
    # Fetch all colors
    colors = Color.objects.all().values(
        'id', 'name', 'RAL_NUMBER', 'added_by__first_name', 'created_at', 'updated_at'
    )

    # Fetch all products
    products = Product.objects.all().values(
        'product_id', 'pid', 'product_name', 'product_description', 'category__name',
        'uom__unit', 'gauge__name', 'finish', 'manufacturer', 'creation_date',
        'last_updated_date', 'price', 'min_price', 'discounted_price', 'sku',
        'barcode', 'quantity_in_stock', 'weight', 'dimensions', 'productType',
        'active_status', 'created_at', 'updated_at'
    )

    response_data = {
        "colors": list(colors),  # Convert queryset to list
        "products": list(products),  # Convert queryset to list
    }

    return JsonResponse(response_data, safe=False)
